﻿Option Strict On
Option Explicit On

Imports System
Imports System.Collections
Imports Microsoft.ReportingServices.DataProcessing
Imports System.IO
Imports System.Xml
Imports System.Security


Public Class XmlDataReader
    Implements IDataReader

    ' die aus dem XML-File eingelesene Tabelle
    Private _DataTable As DataTable
    ' unsere Connection
    Private _connection As XmlConnection = Nothing
    ' unsere aktuelle Zeile
    Friend _iAktuelleZeile As Integer

    '---- um zu verhindern, dass die Benutzer DataReader-Objekte direkt erzeugen,
    '---- wird der Konstruktor mit "friend" als intern deklariert
    Friend Sub New(ByVal cmdText As String)
    End Sub 'New
    '---- Methoden und Eigenschaften von IDataReader

    Public Function Read() As Boolean Implements IDataReader.Read
        ' -- geht zur nächsten Zeile
        If (_DataTable Is Nothing) Then
            Return False ' wenn es keine Datentabelle gibt, ist read ein Misserfolg 
        Else
            ' 
            _iAktuelleZeile += 1
            Return _iAktuelleZeile < _DataTable.Rows.Count
        End If
    End Function 'Read

    Public ReadOnly Property FieldCount() As Integer _
            Implements IDataReader.FieldCount
        ' gibt die Anzahl der Spalten zurück
        Get
            Return _DataTable.Columns.Count
        End Get
    End Property

    Public Function GetName(ByVal i As Integer) As String _
            Implements IDataReader.GetName
        ' Gibt den Namen der i-ten Spalte zurück
        Return _DataTable.Columns(i).ColumnName.ToString
    End Function 'GetName

    Public Function GetFieldType(ByVal i As Integer) As Type _
            Implements IDataReader.GetFieldType
        ' Gibt den Typ der i-ten Spalte zurück
        Return _DataTable.Columns(i).DataType
    End Function 'GetFieldType

    Public Function GetValue(ByVal i As Integer) As [Object] _
            Implements IDataReader.GetValue
        ' Gibt den Value der i-ten Spalte der aktuellen Zeile zurück
        Return _DataTable.Rows(_iAktuelleZeile).ItemArray(i)
    End Function 'GetValue

    Public Function GetOrdinal(ByVal name As String) As Integer _
            Implements IDataReader.GetOrdinal
        ' Ermittelt den Ordinalwert der Spalte
        ' Gibt -1 für "nicht gefunden" zurück
        Return _DataTable.Columns(name).Ordinal
    End Function 'GetOrdinal
    '---- Implementierung der spezifischen Methode


    Friend Sub liesXmlDataDocument(ByVal cmdText As String)
        Dim XmlDataDoc As New XmlDataDocument
        ' -- Schema einlesen

        Dim reader As StreamReader = Nothing
        Try
            reader = New StreamReader(cmdText & ".xsd")
            ' Schemadatei wird gelesen
            XmlDataDoc.DataSet.ReadXmlSchema(reader)
            ' jetzt lesen wir die Daten
            XmlDataDoc.Load(cmdText & ".xml")
        Catch e As Exception
            Throw New InvalidOperationException("Fehler: " & e.ToString())
        Finally
            If Not reader Is Nothing Then
                reader.Close()
            End If

        End Try
        ' uns interessiert nur die erste Tabelle
        _DataTable = XmlDataDoc.DataSet.Tables(0)
        _iAktuelleZeile = -1
    End Sub 'liesXmlDataDocument

    Public Sub Dispose() Implements IDataReader.Dispose
    End Sub 'Dispose

End Class 'XmlDataReader
