﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Drawing.Imaging;
using System.IO;
using System.Text;
using Microsoft.ReportingServices.OnDemandReportRendering;


namespace ixto.Beispiele.ReportingServices
{
    public class CustomReportItem1 : ICustomReportItem
    {
        #region ICustomReportItem Members
        public void GenerateReportItemDefinition(CustomReportItem cri)
        {            
            cri.CreateCriImageDefinition();
            Image polygonImage = (Image)cri.GeneratedReportItem;            
        }

        public void EvaluateReportItemInstance(CustomReportItem cri)
        {
            // Beziehen der Bild-Definition
            Image polygonImage = (Image)cri.GeneratedReportItem;

            // Rendern des Bildes für das Custom Report Item
            polygonImage.ImageInstance.ImageData = DrawImage(cri);            
        }
        #endregion
        private byte[] DrawImage(CustomReportItem customReportItem)
        {
            int dpi = 96;
            int imageWidth                       = (int)(customReportItem.Width.ToInches()  * dpi);
            int imageHeight                      = (int)(customReportItem.Height.ToInches() * dpi);
            System.Drawing.Bitmap image          = new System.Drawing.Bitmap(imageWidth, imageHeight);
            System.Drawing.Graphics graphics     = System.Drawing.Graphics.FromImage(image);
            System.Drawing.Color backgroundColor = customReportItem.Style.BackgroundColor.Value.ToColor();

            #region Benutzerdefinierte Eigenschaften werden ausgelesen
            int penWidth    = (int)LookupCustomProperty(customReportItem.CustomProperties, "Balkenbreite", 8);
            float dataValue = Single.Parse(this.LookupCustomProperty(customReportItem.CustomProperties,
                  "ValueField", "").ToString());
            float average   = Single.Parse(this.LookupCustomProperty(customReportItem.CustomProperties, 
                  "Durchschnitt", "").ToString());
            float max       = Single.Parse(this.LookupCustomProperty(customReportItem.CustomProperties, 
                  "Maximum", "").ToString());

            System.Drawing.Color colorAverage = System.Drawing.Color.FromName(this.LookupCustomProperty 
                 (customReportItem.CustomProperties,"DurchschnittFarbe", "Black").ToString());
            System.Drawing.Color colorCRI = System.Drawing.Color.FromName(this.LookupCustomProperty 
                 (customReportItem.CustomProperties, "BalkenFarbe", "Blue").ToString());
            #endregion
           #region Hintergrundfarbe setzen
            if (backgroundColor == System.Drawing.Color.Transparent || backgroundColor == 
                System.Drawing.Color.Empty)
            {
                backgroundColor = System.Drawing.Color.White;
            }
            graphics.Clear(backgroundColor);
            #endregion

           //Werte für die Berechnung der Grafik
            float distance; //Variable für die Darstellung der abzubildenden Werte (Skalierung, Ausschläge)
            if (max == 0)
            {
                distance = dataValue * imageWidth;
                average  = average   * imageWidth;
            }
            else
            {
                distance = dataValue / max * imageWidth;
                average  = average   / max * imageWidth;
            }
            // Erzeugen von Pen-Objekte
            System.Drawing.Pen penBar     = new System.Drawing.Pen(colorCRI, penWidth);
            System.Drawing.Pen penAverage = new System.Drawing.Pen(colorAverage, 1);
            // Zeichnen der Grafik
            graphics.DrawLine(
              new System.Drawing.Pen(colorCRI, penWidth)
            , 0                  // links
            , imageHeight / 2    // oben
            , distance           // rechts
            , imageHeight / 2    // unten
            );

            // Durchschnitt-Markierung zeichnen
            graphics.DrawLine(
              new System.Drawing.Pen(colorAverage, 1)
            , average
            , 0
            , average
            , imageHeight
            );

            // Erstellen eines Byte-Arrays zur Speicherung der Bilddaten
            MemoryStream stream = new MemoryStream();
            image.Save(stream, ImageFormat.Bmp);
            byte[] imageData = new byte[(int)stream.Length];
            stream.Seek(0, SeekOrigin.Begin);
            stream.Read(imageData, 0, (int)stream.Length);

            return imageData;
        }
        private object LookupCustomProperty(CustomPropertyCollection customProperties, string name,
                object defaultValue)
        {
            object customPropertyValue = defaultValue;

            if (customProperties == null || customProperties.Count == 0)
            {
                return defaultValue;
            }
            CustomProperty customProperty = customProperties[name];
            if (customProperty != null)
            {   // Wenn die Eigenschaft vorhanden ist, wird geprüft, ob diese eine Erweiterung ist.
                // Ist es eine Erweiterung, wird diese evaluiert und zurückgeliefert.
                // Andernfalls wird der Wert für die Eigenschaft direkt zurückgeliefert.
                if (customProperty.Value.IsExpression)
                    customPropertyValue = customProperty.Instance.Value;
                else
                    customPropertyValue = customProperty.Value.Value;
            }
            return customPropertyValue;
        }
    }
}
